/*$Id: d_mosc.cc,v 17.23 2000/05/26 08:01:37 al Exp $ -*- C++ -*-
 * mos model subcircuit functions
 */
#include "e_elemnt.h"
#include "d_mos_base.h"
/*--------------------------------------------------------------------------*/
//	void	EVAL_MOS_Cgb::tr_eval(ELEMENT*)const;
//	void	EVAL_MOS_Cgd::tr_eval(ELEMENT*)const;
//	void	EVAL_MOS_Cgs::tr_eval(ELEMENT*)const;
/*--------------------------------------------------------------------------*/
/* gate capacitors.  Meyer model.  
 * Refs: Antognetti, Divekar, Spice 2 & 3 code
 * final ref was Spice 2g6 code.
 * all except spice 3 agree except for typos and smoothing.  (yup!!)
 * (smoothing is different)
 * Spice 3 ignores substrate voltage, and returns half the value of Cgs
 */
void EVAL_MOS_Cgb::tr_eval(ELEMENT* brh)const
{
  DEV_MOS* p = prechecked_cast<DEV_MOS*>(brh->owner());
  assert(p);
  const MOS_COMMON* c = prechecked_cast<const MOS_COMMON*>(p->common());
  assert(c);
  const SDP_MOS_BASE* b = prechecked_cast<const SDP_MOS_BASE*>(c->sdp);
  assert(b);

  double cap = brh->value();
  if (p->vgst < - b->phi){ 			/* accumulation */
    cap += b->cgate;
  }else if (p->vgst < 0.){			/* depletion */
    cap += b->cgate * (-p->vgst) / b->phi;
  }else{					/* active, overlap only */
  }
  brh->_y0.f1 = cap;
  brh->_y0.f0 = brh->_y0.x * brh->_y0.f1;
  trace3(brh->long_label().c_str(), brh->_y0.x, brh->_y0.f0, brh->_y0.f1);
}
/*--------------------------------------------------------------------------*/
void EVAL_MOS_Cgd::tr_eval(ELEMENT* brh)const
{
  DEV_MOS* p = prechecked_cast<DEV_MOS*>(brh->owner());
  assert(p);
  const MOS_COMMON* c = prechecked_cast<const MOS_COMMON*>(p->common());
  assert(c);
  const SDP_MOS_BASE* b = prechecked_cast<const SDP_MOS_BASE*>(c->sdp);
  assert(b);
  const MODEL_MOS_BASE* m = prechecked_cast<const MODEL_MOS_BASE*>(c->model());
  if (!m) {brh->_y0.f1= brh->_y0.f0 = 0; return;}
  assert(m);

  double mult;
  double vbs;
  if (m->cmodel == 3){
    mult = 1./3.;
    vbs = 0.;
  }else{
    mult = 2./3.;
    vbs = p->vbs;
  }
  double cap = brh->value();			/* start with overlap cap */
  if (p->vgst > 0.  &&  p->vdsat > p->vds){	/* linear */
    double vdbsat = p->vdsat - vbs;
    double vdb    = p->vds   - vbs;
    double ddif   = 2. * vdbsat - vdb;
    cap += mult * b->cgate * (1. - (vdbsat*vdbsat)/(ddif*ddif));
  }						/* else overlap only */
  brh->_y0.f1 = cap;
  brh->_y0.f0 = brh->_y0.x * brh->_y0.f1;
  trace3(brh->long_label().c_str(), brh->_y0.x, brh->_y0.f0, brh->_y0.f1);
}
/*--------------------------------------------------------------------------*/
void EVAL_MOS_Cgs::tr_eval(ELEMENT* brh)const
{
  DEV_MOS* p = prechecked_cast<DEV_MOS*>(brh->owner());
  assert(p);
  const MOS_COMMON* c = prechecked_cast<const MOS_COMMON*>(p->common());
  assert(c);
  const SDP_MOS_BASE* b = prechecked_cast<const SDP_MOS_BASE*>(c->sdp);
  assert(b);
  const MODEL_MOS_BASE* m = prechecked_cast<const MODEL_MOS_BASE*>(c->model());
  if (!m) {brh->_y0.f1= brh->_y0.f0 = 0; return;}
  assert(m);

  double mult;
  double vbs;
  if (m->cmodel == 3){
    mult = 1./3.;
    vbs = 0.;
  }else{
    mult = 2./3.;
    vbs = p->vbs;
  }
  double cap = brh->value();			/* start with overlap cap */
  if (p->vgst > 0.){				/* active */
    if (p->vdsat > p->vds){			/* linear */
      double vdbsat = p->vdsat - vbs;
      double vdb    = p->vds   - vbs;
      double ddif   = 2. * vdbsat - vdb;
      double ndif   = p->vdsat - p->vds;
      cap += mult * b->cgate * (1. - (ndif*ndif)/(ddif*ddif));
    }else{					/* saturation */
      cap += mult * b->cgate;
    }
  }else if (p->vgst > -b->phi/2.){		/* depletion */
    cap += mult * b->cgate * ((p->vgst / (b->phi/2.)) + 1.);
  }else{					/* accum. = overlap only */
  }
  brh->_y0.f1 = cap;
  brh->_y0.f0 = brh->_y0.x * brh->_y0.f1;
  trace3(brh->long_label().c_str(), brh->_y0.x, brh->_y0.f0, brh->_y0.f1);
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
