/*$Id: e_base.cc,v 17.21 2000/05/02 09:32:08 al Exp $ -*- C++ -*-
 * Base class for "cards" in the circuit description file
 */
#include "ap.h"
#include "s__.h"
#include "u_prblst.h"
#include "u_xprobe.h"
#include "constant.h"
#include "e_base.h"
/*--------------------------------------------------------------------------*/
double CKT_BASE::tr_probe_num(CS&)const {return NOT_VALID;}
XPROBE CKT_BASE::ac_probe_ext(CS&)const {return XPROBE(NOT_VALID);}
/*--------------------------------------------------------------------------*/
BSMATRIX<double>  CKT_BASE::aa;
BSMATRIX<double>  CKT_BASE::lu;
BSMATRIX<COMPLEX> CKT_BASE::acx;
int		  CKT_BASE::_devcount = 0;
/*--------------------------------------------------------------------------*/
CKT_BASE::~CKT_BASE()
{
  trace1("~CKT_BASE", _probes);
  PROBE_LISTS::purge(this);
  trace1("", _probes);
  assert(_probes==0);
  --_devcount;
}
/*--------------------------------------------------------------------------*/
double CKT_BASE::probe_num(const std::string& what)const
{
  if (SIM::mode == sAC){
    return ac_probe_num(what);
  }else{
    return tr_probe_num_str(what);
  }
}
/*--------------------------------------------------------------------------*/
double CKT_BASE::tr_probe_num_str(const std::string& what)const
{
  CS cmd(what);
  return tr_probe_num(cmd);
}
/*--------------------------------------------------------------------------*/
double CKT_BASE::ac_probe_num(const std::string& what)const
{
  int length = what.length();
  mod_t modifier = mtNONE;
  bool want_db = false;
  char parameter[BUFLEN+1];
  strcpy(parameter, what.c_str());

  if (length > 2  &&  pmatch(&parameter[length-2], "DB")){
    want_db = true;
    length -= 2;
  }
  if (length > 1){ // selects modifier based on last letter of parameter
    switch (tolower(parameter[length-1])){
      case 'm': modifier = mtMAG;   length--;	break;
      case 'p': modifier = mtPHASE; length--;	break;
      case 'r': modifier = mtREAL;  length--;	break;
      case 'i': modifier = mtIMAG;  length--;	break;
      default:  modifier = mtNONE;		break;
    }
  }
  parameter[length] = '\0'; // chop
  
  CS p(parameter);
  XPROBE xp = ac_probe_ext(p); // chopped form
  if (!xp.OK()){
    CS w(what);
    xp = ac_probe_ext(w); // full form
  }
  return xp(modifier, want_db);
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
